@@ -1,5 +1,5 @@
 /* messages.c - message routines for rrep.
-   Copyright (C) 2011, 2013 Arno Onken <asnelt@asnelt.org>
+   Copyright (C) 2011, 2013, 2022 Arno Onken <asnelt@asnelt.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ print_version ()
 {
   printf ("%s\n\n", PACKAGE_STRING);
   printf (_("\
-Copyright (C) 2011, 2013 %s\n\
+Copyright (C) 2011, 2013, 2019, 2022 %s\n\
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n"), AUTHORS);
@@ -50,7 +50,7 @@ void
 print_usage ()
 {
   printf (_("Usage: %s [OPTION]... PATTERN REPLACEMENT [FILE]...\n"),
-	  program_name);
+          program_name);
 }
 
 /* Prints the help.  */
@@ -66,7 +66,7 @@ contain the special character & to refer to that portion of the pattern space\n\
 which matched, and the special escapes \\1 through \\9 to refer to the\n\
 corresponding matching sub-expressions in PATTERN.\n"));
   printf (_("Example: %s 'hello world' 'Hello, World!' menu.h main.c\n"),
-	  program_name);
+          program_name);
   printf ("\n");
   printf (_("\
 Options:\n\
@@ -101,7 +101,7 @@ Options:\n\
   printf (_("\
 With no FILE, or when FILE is -, read standard input and write to standard\n\
 output.  Exit status is %d if any error occurs, %d otherwise.\n"), EXIT_FAILURE,
-	  EXIT_SUCCESS);
+          EXIT_SUCCESS);
   printf ("\n");
   printf (_("\
 The backup suffix is ~, unless set with --suffix or SIMPLE_BACKUP_SUFFIX.\n\
@@ -134,123 +134,123 @@ rrep_error (const int errcode, const char *file_name)
   switch (errcode)
     {
     case ERR_PROCESS_ARG:
-      fprintf (stderr, _("%s: %s: Could not process argument: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could not process argument: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_PROCESS_DIR:
-      fprintf (stderr, _("%s: %s: Could not process directory: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could not process directory: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_PATTERN:
       fprintf (stderr, _("%s: PATTERN must have at least one character\n"),
-	       program_name);
+               program_name);
       break;
     case ERR_UNKNOWN_ESCAPE:
-      fprintf (stderr, _("%s: %s: Unknown escape sequence in REPLACEMENT\n"),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: unknown escape sequence in REPLACEMENT\n"),
+               program_name, file_name);
       break;
     case ERR_SAVE_DIR:
-      fprintf (stderr, _("%s: Could not save current working directory: "),
-	       program_name);
+      fprintf (stderr, _("%s: could not save current working directory: "),
+               program_name);
       perror (NULL);
       break;
     case ERR_ALLOC_SUFFIX:
-      fprintf (stderr, _("%s: Could not allocate memory for suffix: "),
-	       program_name);
+      fprintf (stderr, _("%s: could not allocate memory for suffix: "),
+               program_name);
       perror (NULL);
       break;
     case ERR_ALLOC_BUFFER:
-      fprintf (stderr, _("%s: Could not allocate memory for buffer: "),
-	       program_name);
+      fprintf (stderr, _("%s: could not allocate memory for buffer: "),
+               program_name);
       perror (NULL);
       break;
-    case ERR_ALLOC_FILEBUFFER: 
-      fprintf (stderr, _("%s: %s: Could not allocate memory for file_buffer: "),
-	       program_name, file_name);
+    case ERR_ALLOC_FILEBUFFER:
+      fprintf (stderr, _("%s: %s: could not allocate memory for file_buffer: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_ALLOC_FILELIST:
-      fprintf (stderr, _("%s: Could not allocate memory for file_list: "),
-	       program_name);
+      fprintf (stderr, _("%s: could not allocate memory for file_list: "),
+               program_name);
       perror (NULL);
       break;
     case ERR_ALLOC_PATHBUFFER:
-      fprintf (stderr, _("%s: %s: Could not allocate memory for next_path: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could not allocate memory for next_path: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_ALLOC_PATTERN:
-      fprintf (stderr, _("%s: Could not allocate memory for pattern: "),
-	       program_name);
+      fprintf (stderr, _("%s: could not allocate memory for pattern: "),
+               program_name);
       perror (NULL);
       break;
     case ERR_ALLOC_REPLACEMENT:
-      fprintf (stderr, _("%s: Could not allocate memory for replacement: "),
-	       program_name);
+      fprintf (stderr, _("%s: could not allocate memory for replacement: "),
+               program_name);
       perror (NULL);
       break;
     case ERR_ALLOC_BACKUP:
       fprintf (stderr, _("\
-%s: %s: Could not allocate memory for backup string: "), program_name,
-	       file_name);
+%s: %s: could not allocate memory for backup string: "), program_name,
+               file_name);
       perror (NULL);
       break;
     case ERR_REALLOC_BUFFER:
-      fprintf (stderr, _("%s: %s: Could not reallocate memory for buffer: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could not reallocate memory for buffer: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_REALLOC_FILEBUFFER:
       fprintf (stderr, _("\
-%s: %s: Could not reallocate memory for file_buffer: "), program_name,
-	       file_name);
+%s: %s: could not reallocate memory for file_buffer: "), program_name,
+               file_name);
       perror (NULL);
       break;
     case ERR_MEMORY:
-      fprintf (stderr, _("%s: %s: Not enough memory to process file: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: not enough memory to process file: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_OPEN_READ:
-      fprintf (stderr, _("%s: %s: Could not open file for reading: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could not open file for reading: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_OPEN_WRITE:
-      fprintf (stderr, _("%s: %s: Could not open file for writing: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could not open file for writing: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_OPEN_DIR:
-      fprintf (stderr, _("%s: Could not open directory: "),
-	       program_name);
+      fprintf (stderr, _("%s: could not open directory: "),
+               program_name);
       perror (NULL);
       break;
     case ERR_READ_FILE:
-      fprintf (stderr, _("%s: %s: Could not read file: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could not read file: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_READ_TEMP:
-      fprintf (stderr, _("%s: %s: Could not read temporary file: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could not read temporary file: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_WRITE_BACKUP:
-      fprintf (stderr, _("%s: %s: Could not write backup file: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could not write backup file: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_OVERWRITE:
-      fprintf (stderr, _("%s: %s: Could not overwrite file: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could not overwrite file: "),
+               program_name, file_name);
       perror (NULL);
       break;
     case ERR_KEEP_TIMES:
-      fprintf (stderr, _("%s: %s: Could keep file times: "),
-	       program_name, file_name);
+      fprintf (stderr, _("%s: %s: could keep file times: "),
+               program_name, file_name);
       perror (NULL);
       break;
     }
@@ -274,7 +274,7 @@ print_regerror (const int errcode, regex_t *compiled)
 void
 print_confirmation (const char *file_name)
 {
-  printf (_("%s: Pattern replaced\n"), file_name);
+  printf (_("%s: pattern replaced\n"), file_name);
 }
 
 /* Prints directory omission.  */
@@ -282,8 +282,8 @@ void
 print_dir_skip (const char *file_name)
 {
   if (!(options & OPT_QUIET))
-    printf (_("%s: %s: Omitting directory\n"), program_name,
-	    file_name);
+    printf (_("%s: %s: omitting directory\n"), program_name,
+            file_name);
 }
 
 /* Prints the simulation message.  */
@@ -292,7 +292,7 @@ print_dry ()
 {
   if (options & OPT_DRY && !(options & OPT_QUIET))
     {
-      printf (_("Simulation mode: No files are modified.\n"));
+      printf (_("Simulation mode: no files are modified.\n"));
       printf (_("PATTERN found in the following files:\n"));
     }
 }
@@ -301,9 +301,9 @@ print_dry ()
 bool
 prompt_user (const char *file_name)
 {
-  /* TRANSLATORS: This is a user prompt. In English the user can answer with y
-     for 'yes' or n for 'no'. [y/n] should be translated as well.  */
-  printf (_("Pattern found in %s. Replace pattern [y/n]? "), file_name);
+  /* TRANSLATORS: this is a user prompt.  In English the user can answer with y
+     for 'yes' or n for 'no'.  [y/n] should be translated as well.  */
+  printf (_("Pattern found in %s.  Replace pattern [y/n]? "), file_name);
 
   return yesno ();
 }
